@@ -5,4 +5,4 @@ sub m1  : Attr {default => 1}
 sub m2 : Attr {default => 2}
 
 
-Object::Simple->end;
+Object::Simple->build_class;
